inline void echolesky_decomposition(MatrixXd & A) 
{
  register int i, j, k;
  register double sum;
  int n=A.rows();	
  for (i = 0; i < n; i++)
  {
    for (j = i; j < n; j++)
    {
      sum = A(i,j);
      for (k = i - 1; k >= 0; k--)
        sum -= A(i,k)*A(j,k);
      if (i == j) 
      {
        if (sum <= 0.0)
				{
          // raise error
			//		print_matrix("A", A, n);
          throw std::runtime_error("The matrix passed to the Cholesky A = L L^T decomposition is not positive definite");
				}
        A(i,i) = sqrt(sum);
      }
      else
        A(j,i) = sum / A(i,i);
    }
    for (k = i + 1; k < n; k++)
      A(i,k) = A(k,i);
  } 
}