#include "eigen_common.h"
#include "lipm_model.h"

lipmModel::lipmModel(){
	num_states = 2;
	num_actions = 1;
	TIMESTEP = 0.05;
	is_linear = true;
	is_control_inequality_constrained = false;
}

void lipmModel::forward_dynamics(double t, Vectord &x, Vectord &u, Vectord &x_next){
	x_next(0) = x(0) + TIMESTEP*x(1);
	x_next(1) = x(1) + TIMESTEP*9.81*(x(0) - u(0));
}

void lipmModel::user_forward_dynamics_derivatives(double t, Vectord &x, Vectord &u, Matrixd &Fx, Matrixd &Fu, Matrixd3D &Fxx, Matrixd3D &Fux, Matrixd3D &Fuu){
	Fx(0,0) = 1;
	Fx(0,1) = TIMESTEP;
	Fx(1,0) = TIMESTEP*9.81;
	Fx(1,1) = 1;

	Fu(0,0) = 0;
	Fu(1,0) = -TIMESTEP*9.81;
}

static double q0 = 1;
static double q1 = 1;
static double r0 = 0.01;

double lipmModel::one_step_cost(double t, Vectord &x, Vectord &u){
	double cost = 0;

	cost = 0.5*x(0)*q0*x(0) + 0.5*x(1)*q1*x(1) + 0.5*u(0)*r0*u(0);

	return cost;
}

void lipmModel::user_one_step_cost_derivatives(double t, Vectord &x, Vectord &u, Vectord &Lx, Vectord &Lu, Matrixd &Lxx, Matrixd &Lux, Matrixd &Luu){

	Lx(0) = q0*x(0);
	Lx(1) = q1*x(1);

	Lu(0) = r0*u(0);
	
	Lxx(0,0) = q0;
	Lxx(1,1) = q1;

	Luu(0,0) = r0;
}

double lipmModel::user_final_cost(double t, Vectord &x, Vectord &u, Vectord &Lx, Matrixd &Lxx){
	double cost = 0;

	cost = 0.5*x(0)*q0*x(0) + 0.5*x(1)*q1*x(1);

	Lx(0) = q0*x(0);
	Lx(1) = q1*x(1);
	
	Lxx(0,0) = q0;
	Lxx(1,1) = q1;

	return cost;

}

void lipmModel::user_control_inequality_constraints(double t, Vectord &x, Vectord&u, Vectord&c, Matrixd &C){
	c(0) = u(0) - 0.1;
	c(1) = -u(0) - 0.1;

	C(0,0) = 1;
	C(1,0) = -1;
}
