#ifndef NONLINEAR_MPC_H
#define NONLINEAR_MPC_H

#include "eigen_common.h"
#include "mpc_model.h"


class NonlinearMPC{

public:
	NonlinearMPC(int _N_TIMESTEPS, double _TIMESTEP);
	~NonlinearMPC();

	void initialize();
	
	void allocate();
	void deallocate();

	void set_model(mpcModel *m);
	
	// optimization functions
	void run(double t0, Vectord &x0);		// given an initial condition, start the optimization
	void forward(double t0, Vectord &x0);	// simulate forward and compute derivatives
	void backward();	// propogate value function backwards
	void forward2();			// iteratively compute new nominal trajectory
	
	//void get_total_cost();
	//void get_state_trajectory();
	//void get_cost_trajectory();
	//void get_control_trajectory();

	double mu_regularization;	// used to regularize Quu
	
private:

	bool is_initialized;
	bool is_model_set;
	bool is_allocated;
	bool is_first_iter;

	int N_TIMESTEPS;
	double TIMESTEP;

	mpcModel *model;

	// all the saved value derivatives and value function trajectories
	Vectord *X;	
	Vectord *U;
	Vectord *dU;
	Vectord *Xhat;	
	Vectord *Uhat;

	double *V;
	Vectord *Vx;
	Matrixd *Vxx;	
	Matrixd *K;

	Matrixd *Fx;
	Matrixd *Fu;
	Matrixd3D *Fxx;
	Matrixd3D *Fux;
	Matrixd3D *Fuu;
		
	Vectord *Lx;	
	Vectord *Lu;
	Matrixd *Lxx;
	Matrixd *Lux;
	Matrixd *Luu;

	Vectord *Qx;
	Vectord *Qu;
	Matrixd *Qxx;
	Matrixd *Qux;
	Matrixd *Quu;

	double cost_nominal;		// cost of the saved nominal trajectory
	double cost_temporary;		// cost of the temp trajectory used in forward2 linesearch
	double cost_expected_change;	// approx. expected change in trajectory cost

};

#endif